/* 
http://www.jibble.org/piespy/

Copyright Paul James Mutton, 2001-2004, http://www.jibble.org/

This file is part of PieSpy.

This software is dual-licensed, allowing you to choose between the GNU
General Public License (GPL) and the www.jibble.org Commercial License.
Since the GPL may be too restrictive for use in a proprietary application,
a commercial license is also provided. Full license information can be
found at http://www.jibble.org/licenses/

$Author: pjm2 $
$Id: BisqMain.java,v 1.34 2004/05/11 17:07:37 pjm2 Exp $

*/

package org.jibble.socnet;

import java.util.*;
import java.awt.image.BufferedImage;
import java.io.*;

/*

Used features:

 Remember a nick that is on a channel:
 
   private void add(Channel, Nick)
 
 Rename a known nick:
 
   private void changeNick(old, new)
 

*/


public class BisqMain {

    public static final String VERSION = "PieSpy 0.4.0";

    public BisqMain(Configuration config) throws IOException {
        this.config = config;
        
        // Prevent construction if the output directory does not exist.
        if (!config.outputDirectory.exists() || !config.outputDirectory.isDirectory()) {
            throw new IOException("Output directory (" + config.outputDirectory + ") does not exist.");
        }
        
    }
    
    private void msg(String channel, String nick, String message)
    {
        //  Analyze a "message" from "sender" at a "channel":
        //  Remove colours first.
        //  Take "actions" as plain messages too (no sender nick in message).
        
        add(channel, nick);
        
        // Pass the message on to the InferenceHeuristics in the channel's Graph.
        String key = channel.toLowerCase();
        Graph graph = (Graph) _graphs.get(key);
        graph.infer(nick, message);
    }
    
    private void createGFX(String channel)
    {
        Iterator graphIt = _graphs.values().iterator();
        while (graphIt.hasNext())
        {
            Graph graph = (Graph) graphIt.next();
            graph.makeNextImage();
        }
     }
    
    private void add(String channel, String nick)
    {
        Node node = new Node(nick);
        String key = channel.toLowerCase();
        
        // Create the Graph for this channel if it doesn't already exist.
        Graph graph = (Graph) _graphs.get(key);
        if (graph == null) {
            if (config.createRestorePoints) {
                graph = readGraph(key);
            }
            if (graph == null) {
                graph = new Graph(channel, config);
            }
            _graphs.put(key, graph);
        }
        
        // Add the Node to the Graph.
        graph.addNode(node);
    }

    private void changeNick(String oldNick, String newNick)
    {
        // Effect the nick change by calling the mergeNode method on all Graphs.
        Iterator graphIt = _graphs.values().iterator();
        while (graphIt.hasNext()) {
            Graph graph = (Graph) graphIt.next();
            Node oldNode = new Node(oldNick);
            Node newNode = new Node(newNick);
            graph.mergeNode(oldNode, newNode);
        }
    }
    
    // Read a serialized graph from disk.
    private Graph readGraph(String channel)
    {
        Graph g = null;
        // Try and see if the graph can be restored from file.
        try {
            String strippedChannel = channel.toLowerCase().substring(1);
            
            File dir = new File(config.outputDirectory, strippedChannel);
            File file = new File(dir, strippedChannel + "-restore.dat");
            ObjectInputStream ois = new ObjectInputStream(new FileInputStream(file));
            String version = (String) ois.readObject();
            if (version.equals(BisqMain.VERSION)) {
                // Only read the object if the file is for the correct version.
                g = (Graph) ois.readObject();
            }
            ois.close();
        }
        catch (Exception e) {
            // Do nothing?
        }
        
        return g;
    }
    
    public void Save()
    {
        Iterator graphIt = _graphs.values().iterator();
        while (graphIt.hasNext()) {
            Graph graph = (Graph) graphIt.next();
            graph.writeGraph();
        }
    }

    public void Stats()
    {
        // Tell the user about the Graphs currently stored.
        Iterator keyIt = _graphs.keySet().iterator();
        while (keyIt.hasNext())
        {
            String key = (String) keyIt.next();   
            Graph graph = (Graph) _graphs.get(key);
            System.out.println(key + ": " + graph.toString());
        }
    }
    
    public static void main(String[] args) throws Exception
    {
        Properties p = new Properties();
        
        String configFile = "./config.ini";
        
        p.load(new FileInputStream(configFile));
        Configuration config = new Configuration(p);
        
        BisqMain bot = new BisqMain(config);
        
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        for(;;)
        {
            String line = br.readLine();
            if(line == null) break;
            
            if(line.startsWith("M "))
            {
                line = line.substring(2).trim();
                StringTokenizer tokenizer = new StringTokenizer(line);
                if (tokenizer.countTokens() >= 3)
                {
                    String channel = tokenizer.nextToken();
                    String nick    = tokenizer.nextToken();
                    String message = line.substring(channel.length()+nick.length()
                                                  + 2+ 1+ 1).trim();
                    //msg: M channel nick message
                    
                    bot.add(channel, nick);
                    bot.msg(channel, nick, message);
                    
                    //System.out.println("M:c("+channel+")n("+nick+")m("+message+")");
                    //bot.createGFX(channel);
                }
            }
            else if(line.startsWith("N "))
            {
                line = line.substring(2).trim(); 
                
                StringTokenizer tokenizer = new StringTokenizer(line);
                if (tokenizer.countTokens() >= 2)
                {
                    String oldnick = tokenizer.nextToken();
                    String newnick = tokenizer.nextToken();
                    //nick: N oldnick newnick

                    //System.out.println("N:n1("+oldnick+")n2("+newnick+")");
                    //bot.changeNick(oldnick, newnick);
                }
            }
            else if(line.startsWith("K "))
            {
                line = line.substring(2).trim(); 
                StringTokenizer tokenizer = new StringTokenizer(line);
                if (tokenizer.countTokens() >= 2)
                {
                    String channel = tokenizer.nextToken();
                    String nick    = tokenizer.nextToken();
                    //know: K channel nick

                    System.out.println("K:c("+channel+")n("+nick+")");

                    bot.add(channel, nick);
                }
            }
            else
            {
                bot.Stats();
            }
        }
        
        bot.Save();
        
        return;
    }
    
    
    // HashMap of String -> Graph objects.
    private HashMap _graphs = new HashMap();

    private Configuration config;
    
}